set ask
set askcc
set crt=24
set dot
set EDITOR=/usr/bin/vi
set hold
set keepsave
set metoo
set PAGER=/usr/bin/more
set SHELL=/bin/sh
set VISUAL=/usr/bin/vi
set indentprefix="> "

ignore apparently-to date errors-to from id in-reply-to \
       message-id precedence received references remailed-date \
       remailed-from return-path sent-by status via
